@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[124]:
+# In[146]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[125]:
+# In[147]:
 
 
-version = '0.67'
+version = '0.69'
 
 import re
 import tempfile
@@ -33,7 +33,7 @@ import time
 
 # # Helper Functions
 
-# In[126]:
+# In[148]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[127]:
+# In[149]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[128]:
+# In[150]:
 
 
 def testPrint(*args):
@@ -64,13 +64,13 @@ def testPrint(*args):
 
 # # Timing
 
-# In[129]:
+# In[151]:
 
 
 timeSinceLastCallDict = {}
 
 
-# In[130]:
+# In[152]:
 
 
 def timeSinceLastCall(name):
@@ -83,7 +83,7 @@ def timeSinceLastCall(name):
     return name + ": " + time_s
 
 
-# In[131]:
+# In[153]:
 
 
 if is_testing():
@@ -92,7 +92,7 @@ if is_testing():
 
 # # Requiremtns
 
-# In[132]:
+# In[154]:
 
 
 requirements = []
@@ -124,7 +124,7 @@ import yaml
 
 # # Strings
 
-# In[133]:
+# In[155]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -154,7 +154,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[134]:
+# In[156]:
 
 
 def strUuid4():
@@ -165,14 +165,14 @@ def strUuid4():
     return uuid
 
 
-# In[135]:
+# In[157]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[136]:
+# In[158]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -184,7 +184,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[137]:
+# In[159]:
 
 
 if is_testing():
@@ -193,7 +193,7 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
-# In[138]:
+# In[160]:
 
 
 def strMd5Pickle4(obj):
@@ -203,14 +203,14 @@ def strMd5Pickle4(obj):
     return md5.hexdigest()
 
 
-# In[139]:
+# In[161]:
 
 
 if is_testing():
     print(strMd5Pickle4(['a', 'b']))
 
 
-# In[140]:
+# In[162]:
 
 
 def strMd5String(s):
@@ -219,7 +219,7 @@ def strMd5String(s):
     return md5.hexdigest()
 
 
-# In[141]:
+# In[163]:
 
 
 if is_testing():
@@ -228,7 +228,7 @@ if is_testing():
 
 # # Process / Thread
 
-# In[142]:
+# In[164]:
 
 
 def mapThread(func, args, n=8):
@@ -251,14 +251,14 @@ if is_testing():
 
 # # Lists
 
-# In[143]:
+# In[165]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[144]:
+# In[166]:
 
 
 if is_testing():
@@ -267,7 +267,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[145]:
+# In[167]:
 
 
 def liSplit(a, number_of_splits):
@@ -275,7 +275,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[146]:
+# In[168]:
 
 
 if is_testing():
@@ -287,7 +287,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[147]:
+# In[169]:
 
 
 def liJoin(l):
@@ -297,14 +297,14 @@ def liJoin(l):
     return out
 
 
-# In[148]:
+# In[170]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[149]:
+# In[171]:
 
 
 def liFlatten(l):
@@ -321,7 +321,7 @@ def liFlatten(l):
         return l
 
 
-# In[150]:
+# In[172]:
 
 
 if is_testing():
@@ -331,7 +331,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[151]:
+# In[173]:
 
 
 regex_cache = {}
@@ -341,35 +341,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[152]:
+# In[174]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[153]:
+# In[175]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[154]:
+# In[176]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[155]:
+# In[177]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[156]:
+# In[178]:
 
 
 def lCsv(l, force_oneline=False):
@@ -385,21 +385,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[157]:
+# In[179]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[158]:
+# In[180]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[159]:
+# In[181]:
 
 
 if is_testing():
@@ -408,7 +408,7 @@ if is_testing():
 
 # # Text files
 
-# In[160]:
+# In[182]:
 
 
 def txtread(path):
@@ -429,7 +429,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[161]:
+# In[183]:
 
 
 if is_testing():
@@ -438,21 +438,21 @@ if is_testing():
 
 # # YAML files
 
-# In[162]:
+# In[184]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj))
 
 
-# In[163]:
+# In[185]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[164]:
+# In[186]:
 
 
 if is_testing():
@@ -465,7 +465,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[165]:
+# In[187]:
 
 
 regex_cache = {}
@@ -477,35 +477,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[166]:
+# In[188]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[167]:
+# In[189]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[168]:
+# In[190]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[169]:
+# In[191]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[170]:
+# In[192]:
 
 
 if is_testing():
@@ -514,7 +514,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[171]:
+# In[193]:
 
 
 import warnings
@@ -538,7 +538,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[172]:
+# In[194]:
 
 
 def gsGsToHttp(path):
@@ -548,7 +548,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[173]:
+# In[195]:
 
 
 def gsLs(path):
@@ -558,7 +558,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[174]:
+# In[196]:
 
 
 def gsExists(path):
@@ -568,7 +568,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[175]:
+# In[197]:
 
 
 def gsUploadFile(gsPath, path):
@@ -576,7 +576,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[176]:
+# In[198]:
 
 
 def gsTxtread(path):
@@ -588,7 +588,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[177]:
+# In[199]:
 
 
 def gsTxtwrite(path, s):
@@ -596,7 +596,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[178]:
+# In[200]:
 
 
 def gsImwrite(path, img):
@@ -605,7 +605,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[179]:
+# In[201]:
 
 
 def gsImread(path):
@@ -614,7 +614,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[180]:
+# In[202]:
 
 
 def gsPklWrite(path, obj):
@@ -622,7 +622,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[181]:
+# In[203]:
 
 
 def gsPklRead(path):
@@ -632,7 +632,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[182]:
+# In[204]:
 
 
 def pklwrite(path, obj):
@@ -643,7 +643,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[183]:
+# In[205]:
 
 
 def pklread(path):
@@ -654,7 +654,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[184]:
+# In[206]:
 
 
 if is_testing():
@@ -667,7 +667,7 @@ if is_testing():
 
 # # Images
 
-# In[185]:
+# In[207]:
 
 
 def imread(path):
@@ -701,7 +701,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[186]:
+# In[208]:
 
 
 def imshow(array, scale=1):
@@ -716,7 +716,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[187]:
+# In[209]:
 
 
 if is_testing():
@@ -724,14 +724,14 @@ if is_testing():
     imshow(img)
 
 
-# In[188]:
+# In[210]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[189]:
+# In[211]:
 
 
 def imscaleNN(img, s):
@@ -741,7 +741,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[190]:
+# In[212]:
 
 
 if is_testing():
@@ -760,7 +760,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[191]:
+# In[213]:
 
 
 def imRepeat(img, height, width):
@@ -774,7 +774,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[192]:
+# In[214]:
 
 
 if is_testing():
@@ -782,33 +782,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[193]:
+# In[215]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[194]:
+# In[216]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[195]:
+# In[217]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[196]:
+# In[218]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[197]:
+# In[219]:
 
 
 def imAddNoiseGauss(img, std):
@@ -820,7 +820,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[198]:
+# In[220]:
 
 
 def imJpgDegradation(img, quality):
@@ -833,7 +833,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[199]:
+# In[221]:
 
 
 def imCropCenter(img, size):
@@ -848,35 +848,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[200]:
+# In[222]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[201]:
+# In[223]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[202]:
+# In[224]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[203]:
+# In[225]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[204]:
+# In[226]:
 
 
 def imCropRandom(img, size):
@@ -885,13 +885,13 @@ def imCropRandom(img, size):
     h_start = np.random.randint(0, h-size)
     h_end = h_start + size
 
-    w_start = np.random.randint(0, h-size)
+    w_start = np.random.randint(0, w-size)
     w_end = w_start + size
 
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[205]:
+# In[227]:
 
 
 if is_testing(): 
@@ -899,7 +899,7 @@ if is_testing():
     imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
 
 
-# In[206]:
+# In[228]:
 
 
 def imGallery(imgs, pad=0):
@@ -930,7 +930,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[207]:
+# In[229]:
 
 
 if is_testing():
@@ -938,49 +938,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[208]:
+# In[230]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[209]:
+# In[231]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[210]:
+# In[232]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[211]:
+# In[233]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[212]:
+# In[234]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[213]:
+# In[235]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[214]:
+# In[236]:
 
 
 if is_testing():
@@ -990,14 +990,14 @@ if is_testing():
 
 # # Files
 
-# In[215]:
+# In[237]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[216]:
+# In[238]:
 
 
 if is_testing():
@@ -1026,7 +1026,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[217]:
+# In[239]:
 
 
 def fiRemove(path):
@@ -1054,7 +1054,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[218]:
+# In[240]:
 
 
 if is_testing():
@@ -1086,7 +1086,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[219]:
+# In[241]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1101,7 +1101,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[220]:
+# In[242]:
 
 
 def fiJoin(*args):
@@ -1113,7 +1113,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[221]:
+# In[243]:
 
 
 def jnNbFullWidth():
@@ -1127,7 +1127,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[222]:
+# In[244]:
 
 
 def jnImageRenderingPixelated():
@@ -1139,7 +1139,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[223]:
+# In[245]:
 
 
 def jnDefault():
@@ -1157,7 +1157,7 @@ if is_testing():
 
 
 
-# In[224]:
+# In[246]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1170,7 +1170,7 @@ if is_testing():
     
 
 
-# In[225]:
+# In[247]:
 
 
 def jnImgToHtmlB64(img):
@@ -1205,7 +1205,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[226]:
+# In[248]:
 
 
 if is_testing():
@@ -1213,7 +1213,7 @@ if is_testing():
    
 
 
-# In[227]:
+# In[249]:
 
 
 if is_testing():
@@ -1223,7 +1223,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[228]:
+# In[250]:
 
 
 
@@ -1233,7 +1233,7 @@ if is_testing():
 
 # # Shell
 
-# In[229]:
+# In[251]:
 
 
 def shell(cmd):
@@ -1248,7 +1248,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[230]:
+# In[252]:
 
 
 def gitGetHash():
@@ -1258,7 +1258,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[231]:
+# In[253]:
 
 
 def gitGetBranch():
@@ -1268,7 +1268,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[232]:
+# In[254]:
 
 
 def gitGetMessage():
@@ -1278,7 +1278,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[233]:
+# In[255]:
 
 
 def gitGetAllFiles():
@@ -1288,7 +1288,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[234]:
+# In[256]:
 
 
 def gitListByTime(path=None):
@@ -1304,7 +1304,18 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[235]:
+# In[257]:
+
+
+def gitTime(path):
+    _time = lambda x: shell('git log -1 --format="%ad" -- {filename}'.format(filename=x))
+    return _time(path)
+    
+if is_testing():
+    print(gitTime("__init__.py"))
+
+
+# In[258]:
 
 
 def shellSortDirsBySize():
@@ -1316,14 +1327,14 @@ if is_testing():
 
 # # Package Files
 
-# In[236]:
+# In[259]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[237]:
+# In[260]:
 
 
 def shell(cmd):
@@ -1331,7 +1342,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[238]:
+# In[261]:
 
 
 if is_testing():
@@ -1363,7 +1374,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[239]:
+# In[262]:
 
 
 if is_testing():
@@ -1373,7 +1384,7 @@ description-file = README.md
 """
 
 
-# In[240]:
+# In[263]:
 
 
 if is_testing():
@@ -1396,14 +1407,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[241]:
+# In[264]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[242]:
+# In[265]:
 
 
 if is_testing():
@@ -1412,7 +1423,7 @@ if is_testing():
     
 
 
-# In[243]:
+# In[266]:
 
 
 if is_testing():
@@ -1423,7 +1434,7 @@ lx/__init__.py
 """
 
 
-# In[244]:
+# In[267]:
 
 
 if is_testing():
